from .air_pls import AirPls
from .cubic_spline_correction import CubicSplineCorrection
from .linear_correction import LinearCorrection
from .non_negative import NonNegative
from .polynomial_correction import PolynomialCorrection